package org.colinvella.fancyfish.entity.ai;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.entity.fish.FishEntity;
import org.colinvella.fancyfish.logger.ModLogger;

import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.init.Blocks;
import net.minecraft.util.BlockPos;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import scala.util.Random;

public class FishEntityAIFlopAround extends EntityAIBase
{   
    public FishEntityAIFlopAround(FishEntity fishEntity)
    {
    	if (logger == null)
    		logger = FancyFishMod.getLogger();
    	
    	this.fishEntity = fishEntity;
    	this.random = new Random();
    }

    @Override public void resetTask()
    {
    	targetWater = false;
    };
    
    @Override
    public boolean shouldExecute()
    {
        return !this.fishEntity.isInWater();
    }

    @Override
    public void updateTask()
    {
    	this.fishEntity.getJumpHelper().setJumping();

        if (this.fishEntity.getRNG().nextInt(10) == 0 || !this.fishEntity.hasTargetMotion())
        {
        	BlockPos pos = null;
        	if (!targetWater)
        	{
    			World world = fishEntity.worldObj;
    			
        		for (int tries = 0; tries < 4; tries++)
        		{
            		pos = new BlockPos(fishEntity);
            		pos = pos.east(random.nextInt(5) - 2).north(random.nextInt(5) - 2).up();
        			while (world.getBlockState(pos).getBlock() == Blocks.air && pos.getY() > 1)
        				pos = pos.down();
        			
        			targetWater = world.getBlockState(pos).getBlock() == Blocks.water;
        			targetX = pos.getX() + 0.5;
        			targetZ = pos.getZ() + 0.5;
        			if (targetWater)
        				break;
        		}
        	}
        	
        	double targetMotionX = 0;
        	double targetMotionY = 0;
        	double targetMotionZ = 0;
        	
        	if (targetWater)
        	{
        		// jump towards water
        		targetMotionX = targetX - fishEntity.posX;
        		targetMotionZ = targetZ - fishEntity.posZ;
        		
        		double factor = 0.2 / Math.sqrt(targetMotionX * targetMotionX + targetMotionZ * targetMotionZ);
        		targetMotionX *= factor;
        		targetMotionZ *= factor;
        	}
        	else
        	{
	        	// choose random angle in XZ plane
	            float angle = this.fishEntity.getRNG().nextFloat() * (float)Math.PI * 2.0F;
	            
	            targetMotionX = MathHelper.cos(angle) * 0.2F;
	            targetMotionZ = MathHelper.sin(angle) * 0.2F;
        	}

            this.fishEntity.setTargetMotion(targetMotionX, targetMotionY, targetMotionZ);
        }
    }
    
    private FishEntity fishEntity;
    private boolean targetWater;
    private double targetX;
    private double targetZ;
    private Random random;
    
    private static ModLogger logger;
}	